<?php

/*

Plugin Name: Blenderhouse Tools
Plugin URI: https://blenderhousecreative.com
Description: Plugin with custom tools for your website to be used with the Blenderhouse Theme.
Version: 1.2
Author: Blenderhouse Creative
Author URI: https://blenderhousecreative.com/

*/

/**
 * 
 * TABLE OF CONTENTS:
 * 
 * NOTE: Remove individual items in TOC and create broader sections. Admin, ACF, White Labeling, CPT ...
 * 
 * 
 * # Enqueue stylesheet (Admin menu seperators)
 * # Customize admin menu order
 * # Rename menu items
 * # Remove menu items globally
 * # Remove menu items (for web dev) from clients (and below) role
 * # Add client role
 * # Custom Post Type for TEMPLATE (Can use cpt ui to generate a template for others for now)
 * # Replace "howdy" with alternative greeting
 * # Replace footer text in admin
 * # ACF - Give sections a title, this adds the Column ID field to the sections title bar
 * # Remove "Append Ninja Forms" from sidebar
 * # ACF options page
 * # Removes buttons from the first row of the tiny mce editor
 * # Removes buttons from the second row (kitchen sink) of the tiny mce editor
 * # Remove customizer options from Appearance Menu
 * # White Label - Login Logo & color
 * # Dynamically generate styles, taking variables/css from /inc/custom-styles.php and saving to /css/custom-styles.css
 * # ACF - Dynamicaly adds global colors (repeater) from options Menu, to the select choices for theme color options.
 * # Custom Dashboard Widgets
 
 * # Update checker



 */


/**
 * 
 * # Enqueue stylesheet (Admin menu seperators)
 *
 */
function add_my_stylesheet() {
	wp_enqueue_style( 'myCSS', plugins_url( 'blenderhouse-tools.css', __FILE__ ) );
}

add_action( 'admin_print_styles', 'add_my_stylesheet' );


/**
 * 
 * # Customize admin menu order
 *
 */
// CUSTOMIZE ADMIN MENU ORDER
function custom_menu_order( $menu_ord ) {
	if ( !$menu_ord ) return true;
	return array(
		'index.php', // Dashboard

		'edit.php?post_type=page', // Pages
		'edit.php', // Posts
		'ninja-forms', // ninja-forms
		'acf-options', // ACF Options
		'upload.php', // Media

		'edit.php?post_type=template', // Custom Post Type 2

		'edit-comments.php', // Comments

		'themes.php', // Appearance
		'plugins.php', // Plugins
		'users.php', // Users
		'tools.php', // Tools
		'options-general.php', // Settings

		'backwpup', // Back WP Up
		'itsec', // iThemes
		'Wordfence', // Wordfence
		'smush', // Smush
		'meowapps-main-menu', // Retina Iamge Plugin
		'wpseo_dashboard', // Yoast

		'edit.php?post_type=acf-field-group', // ACF
		'edit.php?post_type=rmp_menu', // Responsive Menu
		'ai1wm_export', // Migration

		'separator1', // Separator
		'separator2', // Separator
		'separator-last', // Separator


	);
}
add_filter( 'custom_menu_order', 'custom_menu_order' );
add_filter( 'menu_order', 'custom_menu_order' );


/**
 * 
 * # Rename menu items
 *
 */
/*add_filter( 'gettext', 'change_post_to_article' );
function change_post_to_article( $translated ) 
{  
    $translated = str_replace( 'Smush', 'Img Optimizer', $translated );    
    return $translated;
}*/


/**
 * 
 * # Remove menu items globally
 *
 */
function delete_menu_items_globally() {
	remove_menu_page( 'edit-comments.php' );
	remove_submenu_page( 'themes.php', 'widgets.php' );
	/* Ninja Forms */
	remove_submenu_page( 'ninja-forms', 'ninja-forms#apps' );
	/* ACF Extended */
	remove_submenu_page( 'tools.php', 'edit.php?post_type=acfe-dpt' );
	remove_submenu_page( 'tools.php', 'edit.php?post_type=acfe-dt' );
}
add_action( 'admin_init', 'delete_menu_items_globally' );


/**
 * 
 * # Remove menu items (for web dev) from clients role
 *
 */
function delete_menu_items() {
	if ( current_user_can( 'client' ) ) {
		remove_menu_page( 'edit.php?post_type=acf-field-group' );
		remove_menu_page( 'edit.php?post_type=rmp_menu' );
		remove_menu_page( 'ai1wm_export' );
	}
}
add_action( 'admin_init', 'delete_menu_items' );


/**
 * 
 * # Add client role
 *
 */
add_role( 'client', 'Client', get_role( 'administrator' )->capabilities );


/**
 *
 * # Custom Post Type for TEMPLATE
 *
 */
function cptui_register_my_cpts_template() {

	/**
	 * Post Type: Template.
	 */

	$labels = [
		"name" => __( "Template", "custom-post-type-ui" ),
		"singular_name" => __( "Template", "custom-post-type-ui" ),
		"menu_name" => __( "Templates", "custom-post-type-ui" ),
		"all_items" => __( "All Templates", "custom-post-type-ui" ),
		"add_new" => __( "Add new", "custom-post-type-ui" ),
		"add_new_item" => __( "Add new Template", "custom-post-type-ui" ),
		"edit_item" => __( "Edit Template", "custom-post-type-ui" ),
		"new_item" => __( "New Template", "custom-post-type-ui" ),
		"view_item" => __( "View Template", "custom-post-type-ui" ),
		"view_items" => __( "View Template", "custom-post-type-ui" ),
		"search_items" => __( "Search Template", "custom-post-type-ui" ),
		"not_found" => __( "No Template found", "custom-post-type-ui" ),
		"not_found_in_trash" => __( "No Template found in trash", "custom-post-type-ui" ),
		"parent" => __( "Parent Template:", "custom-post-type-ui" ),
		"featured_image" => __( "Featured image for this Template", "custom-post-type-ui" ),
		"set_featured_image" => __( "Set featured image for this Template", "custom-post-type-ui" ),
		"remove_featured_image" => __( "Remove featured image for this Template", "custom-post-type-ui" ),
		"use_featured_image" => __( "Use as featured image for this Template", "custom-post-type-ui" ),
		"archives" => __( "Template archives", "custom-post-type-ui" ),
		"insert_into_item" => __( "Insert into Template", "custom-post-type-ui" ),
		"uploaded_to_this_item" => __( "Upload to this Template", "custom-post-type-ui" ),
		"filter_items_list" => __( "Filter Template list", "custom-post-type-ui" ),
		"items_list_navigation" => __( "Template list navigation", "custom-post-type-ui" ),
		"items_list" => __( "Template list", "custom-post-type-ui" ),
		"attributes" => __( "Template attributes", "custom-post-type-ui" ),
		"name_admin_bar" => __( "Template", "custom-post-type-ui" ),
		"item_published" => __( "Template published", "custom-post-type-ui" ),
		"item_published_privately" => __( "Template published privately.", "custom-post-type-ui" ),
		"item_reverted_to_draft" => __( "Template reverted to draft.", "custom-post-type-ui" ),
		"item_scheduled" => __( "Template scheduled", "custom-post-type-ui" ),
		"item_updated" => __( "Template updated.", "custom-post-type-ui" ),
		"parent_item_colon" => __( "Parent Template:", "custom-post-type-ui" ),
	];

	$args = [
		"label" => __( "Template", "custom-post-type-ui" ),
		"labels" => $labels,
		"description" => "",
		"public" => true,
		"publicly_queryable" => true,
		"show_ui" => true,
		"show_in_rest" => true,
		"rest_base" => "",
		"rest_controller_class" => "WP_REST_Posts_Controller",
		"has_archive" => false,
		"show_in_menu" => true,
		"show_in_nav_menus" => true,
		"delete_with_user" => false,
		"exclude_from_search" => false,
		"capability_type" => "post",
		"map_meta_cap" => true,
		"hierarchical" => false,
		"can_export" => false,
		"rewrite" => [ "slug" => "template", "with_front" => true ],
		"query_var" => true,
		"supports" => [ "title", "editor", "thumbnail" ],
		"show_in_graphql" => false,
	];

	register_post_type( "template", $args );
}

add_action( 'init', 'cptui_register_my_cpts_template' );


/**
 * 
 * # Replace "howdy" with alternative greeting
 *
 */
add_filter( 'admin_bar_menu', 'replace_wordpress_howdy', 25 );

function replace_wordpress_howdy( $wp_admin_bar ) {
	$my_account = $wp_admin_bar->get_node( 'my-account' );
	$newtext = str_replace( 'Howdy,', 'Welcome,', $my_account->title );
	$wp_admin_bar->add_node( array(
		'id' => 'my-account',
		'title' => $newtext,
	) );
}


/**
 * 
 * # Replace footer text in admin
 *
 */
function modify_admin_footer() {
	echo '<span id="footer-thankyou">Developed by <a href="http://www.blenderhousecreative.com" target="_blank" rel="noopener noreferrer">Blenderhouse Creative</a>.</span>';
}
add_filter( 'admin_footer_text', 'modify_admin_footer' );


/**
 * 
 * # ACF - Give sections a title, this adds the Column ID field to the sections title bar
 *
 */

add_filter( 'acf/fields/flexible_content/layout_title', function ( $title ) {
	$ret = $title;
	if ( $custom_title = get_sub_field( 'column_id' ) ) {
		$ret = sprintf( $title . ": " . "<strong>" . $custom_title . "</strong>" );
	}
	return $ret;
} );


/**
 * # Remove "Append Ninja Forms" from sidebar
 */
add_action( 'add_meta_boxes', function () {
	remove_meta_box( 'nf_admin_metaboxes_appendaform', [ 'page', 'post' ], 'side' );
}, 99 );


/**
 * # ACF options page
 */
if ( function_exists( 'acf_add_options_page' ) ) {
	acf_add_options_page();
	
	// acf_add_options_sub_page('General');
	// acf_add_options_sub_page('Header');
	// acf_add_options_sub_page('Footer');
}


/**
 * # Removes buttons from the first row of the tiny mce editor
 */
add_filter( 'mce_buttons', 'jivedig_remove_tiny_mce_buttons_from_editor' );

function jivedig_remove_tiny_mce_buttons_from_editor( $buttons ) {
	$remove_buttons = array(
		// 'bold',
		//  'italic',
		// 'strikethrough',
		// 'bullist',
		// 'numlist',
		// 'blockquote',
		// 'hr', // horizontal line
		// 'alignleft',
		// 'aligncenter',
		// 'alignright',
		// 'link',
		// 'unlink',
		// 'wp_more', // read more link
		// 'spellchecker',
		// 'dfw', // distraction free writing mode
		// 'wp_adv', // kitchen sink toggle (if removed, kitchen sink will always display)
	);
	foreach ( $buttons as $button_key => $button_value ) {
		if ( in_array( $button_value, $remove_buttons ) ) {
			unset( $buttons[ $button_key ] );
		}
	}
	return $buttons;
}


/**
 * # Removes buttons from the second row (kitchen sink) of the tiny mce editor
 */
add_filter( 'mce_buttons_2', 'jivedig_remove_tiny_mce_buttons_from_kitchen_sink' );

function jivedig_remove_tiny_mce_buttons_from_kitchen_sink( $buttons ) {
	$remove_buttons = array(
		// 'formatselect',  format dropdown menu for <p>, headings, etc
		//  'underline',
		//  'alignjustify',
		'forecolor', // text color
		// 'pastetext', // paste as text
		// 'removeformat', // clear formatting
		// 'charmap', // special characters
		'outdent',
		'indent',
		// 'undo',
		// 'redo',
		// 'wp_help', // keyboard shortcuts
	);
	foreach ( $buttons as $button_key => $button_value ) {
		if ( in_array( $button_value, $remove_buttons ) ) {
			unset( $buttons[ $button_key ] );
		}
	}
	return $buttons;
}


/**
 * # Remove customizer options from Appearance Menu
 */
function emersonthis_remove_customizer_options( $wp_customize ) {
	$wp_customize->remove_section( 'static_front_page' );
	// $wp_customize->remove_section( 'title_tagline' );
	$wp_customize->remove_section( 'colors' );
	$wp_customize->remove_section( 'header_image' );
	$wp_customize->remove_section( 'background_image' );
	$wp_customize->remove_section( 'nav' );
	$wp_customize->remove_section( 'themes' );
	$wp_customize->remove_section( 'featured_content' );
}
add_action( 'customize_register',
	'emersonthis_remove_customizer_options',
	30 );


/**
 * # White Label - Login Logo & color
 */

function qot_login() {

?>
<style type="text/css">
#login h1 a, .login h1 a {
		background-image: url('<?php the_field('logo', 'option') ?>');
		height: auto;
		width: 200px;
		background-size: 200px auto;
		background-repeat: no-repeat;
		padding-bottom: 30px;
}
html {
		
	}
body {  
   background: <?php echo get_field('login_screen_background_color', 'option') ?>; 
   background-image: url('<?php echo get_field('login_screen_background_image', 'option') ?>'); 
	background-size: cover;
}
body a {  
	color: <?php echo get_field('login_screen_font_color', 'option') ?> !important;
	}
#login {
    padding: 8% 0 0;
}
.login form, p.message {  
	background-color: rgba(255, 255, 255, .8) !important;
	}
.wp-core-ui .button-primary {
	background: <?php echo get_field('login_screen_background_color', 'option') ?> !important; 	
	}
</style>
<?php }
add_action('login_head', 'qot_login');


/**
 * # Dynamically generate styles, taking variables/css from /inc/custom-styles.php and saving to /css/custom-styles.css
 */

function generate_options_css() {
    $ss_dir = get_stylesheet_directory();
    ob_start(); // Capture all output into buffer
    require($ss_dir . '/inc/dynamic-styles.php'); // Grab the custom-style.php file
    $css = ob_get_clean(); // Store output in a variable, then flush the buffer
    file_put_contents($ss_dir . '/css/dynamic-styles.css', $css, LOCK_EX); // Save it as a css file
}
add_action( 'acf/save_post', 'generate_options_css', 20 ); //Parse the output and write the CSS file on post save (thanks Esmail Ebrahimi)


/**
 * # ACF - Dynamicaly adds global colors (repeater) from options Menu, to the select choices for theme color options.
 */

function acf_load_color_field_choices( $field ) {
    
    // reset choices
    $field['choices'] = array();

    // if has rows
    if( have_rows('global_colors_repeater', 'option') ) {
        
        // while has rows
        while( have_rows('global_colors_repeater', 'option') ) {
            
            // instantiate row
            the_row();
            
            // vars
            $value = get_sub_field('global_color');
            $label = get_sub_field('label');

            // append to choices
            $field['choices'][ $value ] = $label;
        }
    }
    // return the field
    return $field;
    
}

add_filter('acf/load_field/name=background_color', 'acf_load_color_field_choices');
add_filter('acf/load_field/name=text_color', 'acf_load_color_field_choices');

add_filter('acf/load_field/name=h_tags_color', 'acf_load_color_field_choices');
add_filter('acf/load_field/name=h1_color', 'acf_load_color_field_choices');
add_filter('acf/load_field/name=h2_color', 'acf_load_color_field_choices');
add_filter('acf/load_field/name=h3_color', 'acf_load_color_field_choices');
add_filter('acf/load_field/name=h4_color', 'acf_load_color_field_choices');

add_filter('acf/load_field/name=link_color', 'acf_load_color_field_choices');
add_filter('acf/load_field/name=link_color_hover_state', 'acf_load_color_field_choices');

add_filter('acf/load_field/name=button_color', 'acf_load_color_field_choices');
add_filter('acf/load_field/name=button_color_hover_state', 'acf_load_color_field_choices');
add_filter('acf/load_field/name=button_text_color', 'acf_load_color_field_choices');
add_filter('acf/load_field/name=button_text_color_hover_state', 'acf_load_color_field_choices');

add_filter('acf/load_field/name=button_border_color', 'acf_load_color_field_choices');
add_filter('acf/load_field/name=button_border_color_hover_state', 'acf_load_color_field_choices');

add_filter('acf/load_field/name=accordion_title_text_color', 'acf_load_color_field_choices');
add_filter('acf/load_field/name=accordion_title_background_color', 'acf_load_color_field_choices');
add_filter('acf/load_field/name=accordion_content_text_color', 'acf_load_color_field_choices');
add_filter('acf/load_field/name=accordion_content_background_color', 'acf_load_color_field_choices');


























/**
 * # Custom Dashboard Widgets
 */

add_action( 'wp_dashboard_setup', 'register_first_custom_dashboard_widget' );
function register_first_custom_dashboard_widget() {
    wp_add_dashboard_widget(
        'my_first_custom_dashboard_widget',
        'My Custom Dashboard Widget',
        'my_first_custom_dashboard_widget_display'
    );
}

function my_first_custom_dashboard_widget_display() {
    echo '<h2><a href="#">Add/Edit Pages</a></h2>';
}


add_action( 'wp_dashboard_setup', 'register_second_custom_dashboard_widget' );
function register_second_custom_dashboard_widget() {
    wp_add_dashboard_widget(
        'my_second_custom_dashboard_widget',
        'My secondCustom Dashboard Widget',
        'my_second_custom_dashboard_widget_display'
    );
}

function my_second_custom_dashboard_widget_display() {
    echo '<h2><a href="#">Add/Edit Posts</a></h2>';
}


/**
 * Add a new dashboard widget.
 */
 // Mind this opening PHP tag

/**
 *	Adds hidden content to admin_footer, then shows with jQuery, and inserts after welcome panel
 *
 *	@author Ren Ventura <EngageWP.com>
 *	@see http://www.engagewp.com/how-to-create-full-width-dashboard-widget-wordpress
 */
add_action( 'admin_footer', 'rv_custom_dashboard_widget' );
function rv_custom_dashboard_widget() {

	
	



	// Bail if not viewing the main dashboard page
	if ( get_current_screen()->base !== 'dashboard' ) {
		return;
	}

	?>



	




	<div id="custom-id" class="welcome-panel" style="display: none;">
		<div class="welcome-panel-content">
			
			
			<div class="container">
		<div class="row">
			<div class="col">Welcome to your new Website!</div>
		</div>
		<div class="row">
			<div class="col">Problems/Questions? Submit a ticket to Blenderhouse. Hourly could rates apply. Ask about our maintenance plans</div>
			<div class="col">Ask about our maintenance plans</div>
		</div>
	</div>
			
			
	
		</div>
	</div>
	<script>
		jQuery(document).ready(function($) {
			$('#welcome-panel').after($('#custom-id').show());
		});
	</script>

<?php }



















/**
 * # Update checker based on https://rudrastyh.com/wordpress/self-hosted-plugin-update.html
 */

defined( 'ABSPATH' ) || exit;


if( ! class_exists( 'blenderhouseUpdateChecker' ) ) {

	class blenderhouseUpdateChecker{

		public $plugin_slug;
		public $version;
		public $cache_key;
		public $cache_allowed;

		public function __construct() {

			$this->plugin_slug = plugin_basename( __DIR__ );
			$this->version = '1.2';
			$this->cache_key = 'blenderhouse_custom_upd';
			$this->cache_allowed = false;

			add_filter( 'plugins_api', array( $this, 'info' ), 20, 3 );
			add_filter( 'site_transient_update_plugins', array( $this, 'update' ) );
			add_action( 'upgrader_process_complete', array( $this, 'purge' ), 10, 2 );

		}

		public function request(){

			$remote = get_transient( $this->cache_key );

			if( false === $remote || ! $this->cache_allowed ) {

				$remote = wp_remote_get(
					'https://plugin.bhcreative.com/info.json',
					array(
						'timeout' => 10,
						'headers' => array(
							'Accept' => 'application/json'
						)
					)
				);

				if(
					is_wp_error( $remote )
					|| 200 !== wp_remote_retrieve_response_code( $remote )
					|| empty( wp_remote_retrieve_body( $remote ) )
				) {
					return false;
				}

				set_transient( $this->cache_key, $remote, DAY_IN_SECONDS );

			}

			$remote = json_decode( wp_remote_retrieve_body( $remote ) );

			return $remote;

		}


		function info( $res, $action, $args ) {

			// print_r( $action );
			// print_r( $args );

			// do nothing if you're not getting plugin information right now
			if( 'plugin_information' !== $action ) {
				return $res;
			}

			// do nothing if it is not our plugin
			if( $this->plugin_slug !== $args->slug ) {
				return $res;
			}

			// get updates
			$remote = $this->request();

			if( ! $remote ) {
				return $res;
			}

			$res = new stdClass();

			$res->name = $remote->name;
			$res->slug = $remote->slug;
			$res->version = $remote->version;
			$res->tested = $remote->tested;
			$res->requires = $remote->requires;
			$res->author = $remote->author;
			$res->download_link = $remote->download_url;
			$res->trunk = $remote->download_url;
			$res->requires_php = $remote->requires_php;
			$res->last_updated = $remote->last_updated;

			$res->sections = array(
				'description' => $remote->sections->description,
				'installation' => $remote->sections->installation,
				'changelog' => $remote->sections->changelog
			);

			if( ! empty( $remote->banners ) ) {
				$res->banners = array(
					'low' => $remote->banners->low,
					'high' => $remote->banners->high
				);
			}

			return $res;

		}

		public function update( $transient ) {

			if ( empty($transient->checked ) ) {
				return $transient;
			}

			$remote = $this->request();

			if(
				$remote
				&& version_compare( $this->version, $remote->version, '<' )
				&& version_compare( $remote->requires, get_bloginfo( 'version' ), '<' )
				&& version_compare( $remote->requires_php, PHP_VERSION, '<' )
			) {
				$res = new stdClass();
				$res->slug = $this->plugin_slug;
				$res->plugin = plugin_basename( __FILE__ ); // misha-update-plugin/misha-update-plugin.php
				$res->new_version = $remote->version;
				$res->tested = $remote->tested;
				$res->package = $remote->download_url;

				$transient->response[ $res->plugin ] = $res;

	    }

			return $transient;

		}

		public function purge(){

			if (
				$this->cache_allowed
				&& 'update' === $options['action']
				&& 'plugin' === $options[ 'type' ]
			) {
				// just clean the cache when new plugin version is installed
				delete_transient( $this->cache_key );
			}

		}


	}

	new blenderhouseUpdateChecker();

}